#####################################################
# POVer - PovRay Options Project
# Copyright (c) 1999 Poletaev Ernest
# <http://come.to/Poletaev>
# <ufernest@hotmail.com>
# last changes 24.06.1999
#
# Big thank D. Richard Hipp,  Stewart Allen for used in my work:
# Notebook widget for Tcl/Tk, Copyright (C) 1996,1997,1998 D. Richard Hipp
# Visual TCL - A cross-platform application development environment,
# Copyright (C) 1996-1998 Stewart Allen
# And of course Scriptics for Tcl/Tk interpreter

#################################
# GLOBAL VARIABLES
#
global clrButtonFace clrWindow clrButtonShadow clrButtonHighlight
global FileName OutputFile AllowInterrupt ContinueTrace CreateIni
global PauseWhenDone Verbose DrawVistas PartialOutput PartLeft
global PartRight PartTop PartBottom ConstrainAspect ImgSizeX
global ImgSizeY FileFormat ImageSize IneFile Palette Gamma
global DontSave OutputAlpha VideoMode UseGamma UseAnimation
global Clock InitialFrame FinalFrame InitialClock FinalClock CycleAnimation
global FieldRender OddField UseDisplay PovEngine HomeDir IncludeDir
global DefaultDir KeepTemporary widget

set ImageSize 1
set FileName {}
set HomeDir .
set DefaultDir .
set PovEngine {}
set OutputFile {}
set AllowInterrupt 1
set ContinueTrace 0
set CreateIni 0
set PauseWhenDone 1
set Verbose 1
set DrawVistas 0
set PartialOutput 0
set FileFormat 3
set IneFile {}
set Palette 2
set Gamma 2.2
set VideoMode 0
set UseDisplay 1

#################################
# User procedures
#

proc GetHomeDir {} {
global HomeDir
	set home [file dirname [info script]]
	switch [file pathtype $home] {
	    absolute {set HomeDir $home}
	    relative {set HomeDir [file join [pwd] $home]}
	    volumerelative {
	        set curdir [pwd]
	        cd $home
	        set HomeDir [file join [pwd] [file dirname \
	            [file join [lrange [file split $home] 1 end]]]]
	        cd $curdir
		}
	}
}

proc readPreferences {} {
global HomeDir IncludeDir DefaultDir PovEngine
	if {[file exists $HomeDir/.poverrc]} {
    catch {uplevel #0 \
    	[list source $HomeDir/.poverrc]}
    }
}

proc savePreferences {} {
global HomeDir IncludeDir DefaultDir PovEngine
	set OutFile [open $HomeDir/.poverrc w]
	puts $OutFile [concat set PovEngine \{$PovEngine\}]
	puts $OutFile [concat set IncludeDir \{$IncludeDir\}]
	puts $OutFile [concat set DefaultDir \{$DefaultDir\}]
    close $OutFile
}

proc OpenFileDialog {} {
global FileName
set types {
    {{PovRay Script Files}    {.pov}    TEXT}
    {{All Files}        	*           }
 }
 set File [tk_getOpenFile -filetypes $types]
 if {"$File" != ""} {
    set FileName [file nativename $File]
 }
 setOutFile
}

proc IniFileDialog {} {
global IneFile
set types {
    {{PovRay INI Files}    {.ini}    TEXT}
    {{All Files}        	*           }
 }
 set File [tk_getOpenFile -filetypes $types]
 if {"$File" != ""} {
    set IneFile [file nativename $File]
 }
}

proc selectIncludePath {} {
global IncludeDir
set types {
    {{PovRay Script Files}    {.inc}    TEXT}
    {{All Files}        	*           }
 }
 set File [tk_getOpenFile -filetypes $types  -title {Pick any file in Include path}]
 if {"$File" != ""} {
    set IncludeDir [file nativename [file dirname $File]]
 }
}

proc selectDefaultPath {} {
global DefaultDir
set types {
    {{All Files}        	*           }
 }
 set File [tk_getOpenFile -filetypes $types -title {Pick any file in Default path}]
 if {"$File" != ""} {
    set DefaultDir [file nativename [file dirname $File]]
 }
}

proc selectPOVRender {} {
global PovEngine Platform

if {"$Platform"=="windows"} {
   set types {
       {{PovRay engine}  {.exe}  }
       {{All Files}        	*           }
    }
   set binary {pvengine.exe}
 } \
else  {
   set types {
       {{PovRay engine}  *  }
       {{All Files}        	*           }
    }
	set binary {pvengine}
 }
 set File [tk_getOpenFile -filetypes $types -initialfile $binary]
 if {"$File" != ""} {
    set PovEngine [file nativename $File]
 }
}

proc setOutFile {} {
    global OutputFile FileName FileFormat Platform
	set OutputFile $FileName
    switch $FileFormat {
  	  0 {set Ext .tga}
  	  1 {set Ext .tga}
  	  2 {set Ext .ppm}
  	  3 {set Ext .png}
  	  4 {
  	  		switch $Platform {
  	  		   windows {set Ext .bmp}
  	  		   unix {set Ext .ppm}
  	  		   macintosh {set Ext .pic}
  	  		}
  	  	 }
  }
    regsub -all {.pov} $OutputFile $Ext OutputFile
}

proc SaveParm {Parm} {
  if {"$Parm"==""} {return {{}}} \
  else {return $Parm}
}

proc PutBool {Parm} {
	if $Parm {return On} \
	else     {return Off}
}

proc TraceIt {} {
  global FileName OutputFile AllowInterrupt ContinueTrace CreateIni
  global PauseWhenDone Verbose DrawVistas PartialOutput PartLeft
  global PartRight PartTop PartBottom ConstrainAspect ImgSizeX
  global ImgSizeY FileFormat ImageSize IneFile Palette Gamma
  global DontSave OutputAlpha VideoMode UseGamma Clock
  global InitialFrame FinalFrame InitialClock FinalClock CycleAnimation
  global FieldRender OddField UseDisplay UseAnimation PovEngine
  global IncludeDir

  if {"$FileName"==""} return
  set OutFile [open $FileName.ini w]
  puts $OutFile [concat Input_File_Name=$FileName]
  switch $ImageSize {
    0 {puts $OutFile Height=60; puts $OutFile Width=80}
    1 {puts $OutFile Height=240; puts $OutFile Width=320}
    2 {puts $OutFile Height=480; puts $OutFile Width=640}
    3 {
           if $ImgSizeY {puts $OutFile Height=$ImgSizeY}
           if $ImgSizeX {puts $OutFile Width=$ImgSizeX}
      }
  }
  puts $OutFile [concat Test_Abort=[PutBool $AllowInterrupt]]
  puts $OutFile [concat Continue_Trace=[PutBool $ContinueTrace]]
  if {"$CreateIni"==""} [concat Create_Ini=$IneFile]
  puts $OutFile [concat Pause_When_Done=[PutBool $PauseWhenDone]]
  puts $OutFile [concat Verbose=[PutBool $Verbose]]
  puts $OutFile [concat  Draw_Vistas=[PutBool $DrawVistas]]
  if $PartialOutput {
      puts $OutFile Start_Column=$PartLeft
      puts $OutFile End_Column=$PartRight
      puts $OutFile Start_Row=$PartTop
      puts $OutFile End_Row=$PartBottom
  }
  switch $FileFormat {
  	  0 {puts $OutFile +FC}
  	  1 {puts $OutFile +FT}
  	  2 {puts $OutFile +FP}
  	  3 {puts $OutFile +FN}
  	  4 {puts $OutFile +FS}
  }

    if $UseDisplay {
     puts $OutFile Display=On
     switch $VideoMode {
     	  0 {puts $OutFile Video_Mode=0}
    	  1 {
    	  		if {"$CustomVideoMode"!=""} puts $OutFile Video_Mode=$CustomVideoMode \
    	  		else {puts $OutFile Video_Mode=0}
    	  	 }
      }
     switch $Palette {
     	  0 {puts $OutFile Palette=G}
     	  1 {puts $OutFile Palette=H}
     	  2 {puts $OutFile Palette=T}
      }
     if {$UseGamma && "$Gamma"!=""} {puts $OutFile Display_Gamma=$Gamma}
  } else {puts $OutFile Display=Off}

  if $UseAnimation {
	  if {"$Clockputs"!=""} {puts $OutFile Clock=$Clock}
	  if {"$InitialFrame"!=""} {puts $OutFile Initial_Frame=$InitialFrame}
	  if {"$FinalFrame"!=""} {puts $OutFile Final_Frame=$FinalFrame}
	  if {"$InitialClock"!=""} {puts $OutFile Initial_Clock=$InitialClock}
	  if {"$FinalClock"!=""} {puts $OutFile Final_Clock=$FinalClock}
	  if {"$CycleAnimation"!=""} {puts $OutFile Cycle_Animation=$CycleAnimation}
      if {"$FieldRender"!=""} {puts $OutFile Field_Render=$FieldRender}
      if {"$OddField"!=""} {puts $OutFile Odd_Field=$OddField}
   }

  if {"$IncludeDir"!=""} {puts $IncludeDir  Library_Path=$IncludeDir}
  close $OutFile

  exec $PovEngine $FileName.ini &
}

proc OpenIt {} {
}

proc SaveIt {} {
  global FileName OutputFile AllowInterrupt ContinueTrace CreateIni
  global PauseWhenDone Verbose DrawVistas PartialOutput PartLeft
  global PartRight PartTop PartBottom ConstrainAspect ImgSizeX
  global ImgSizeY FileFormat ImageSize IneFile Palette Gamma

  set OutFile [open temp.cfg w]
  puts $OutFile [concat set AllowInterrupt $AllowInterrupt]
  puts $OutFile [concat set ContinueTrace $ContinueTrace]
  puts $OutFile [concat set CreateIni $CreateIni]
  puts $OutFile [concat set PauseWhenDone $PauseWhenDone]
  puts $OutFile [concat set Verbose $Verbose]
  puts $OutFile [concat set DrawVistas $DrawVistas]
  puts $OutFile [concat set PartialOutput $PartialOutput]
  puts $OutFile [concat set PartLeft   [SaveParm $PartLeft]]
  puts $OutFile [concat set PartRight  [SaveParm $PartRight]]
  puts $OutFile [concat set PartTop    [SaveParm $PartTop]]
  puts $OutFile [concat set PartBottom [SaveParm $PartBottom]]
  puts $OutFile [concat set ConstrainAspect $ConstrainAspect]
  puts $OutFile [concat set ImgSizeX [SaveParm $ImgSizeX]]
  puts $OutFile [concat set ImgSizeY [SaveParm $ImgSizeY]]
  puts $OutFile [concat set FileFormat [SaveParm $FileFormat]]
  puts $OutFile [concat set ImageSize [SaveParm $ImageSize]]
  puts $OutFile [concat set IneFile [SaveParm $IneFile]]
  puts $OutFile [concat set Palette [SaveParm $Palette]]
  puts $OutFile [concat set Gamma [SaveParm $Gamma]]
  close $OutFile
}

proc switchPartial {} {
  global PartialOutput clrButtonFace clrWindow
  set base [Notebook:frame .top17.note General]

  if !$PartialOutput {
      $base.fra28.02 configure \
        -background $clrButtonFace -state disabled
      $base.fra28.03 configure \
        -background $clrButtonFace -state disabled
      $base.fra28.04 configure \
        -background $clrButtonFace -state disabled
      $base.fra28.05 configure \
        -background $clrButtonFace -state disabled
  } \
  else {
      $base.fra28.02 configure \
        -background $clrWindow -state normal
      $base.fra28.03 configure \
        -background $clrWindow -state normal
      $base.fra28.04 configure \
        -background $clrWindow -state normal
      $base.fra28.05 configure \
        -background $clrWindow -state normal
  }
}

proc switchIni {} {
  global CreateIni clrButtonFace clrWindow
  set base [Notebook:frame .top17.note General]

  if !$CreateIni {
      $base.fra27.04 configure \
        -background $clrButtonFace -state disabled
  } \
  else {
      $base.fra27.04 configure \
        -background $clrWindow -state normal
  }
}

proc switchGamma {} {
  global UseGamma clrButtonFace clrWindow
  set base [Notebook:frame .top17.note Display]

  if $UseGamma {
      $base.fra31.ent33 configure \
        -background $clrWindow -state normal
  } \
  else {
      $base.fra31.ent33 configure \
        -background $clrButtonFace -state disabled
  }
}

proc switchMosaic {} {
  global MosaicPreview clrButtonFace clrWindow
  set base [Notebook:frame .top17.note Display]

  if !$MosaicPreview {
      $base.fra31.ent36 configure \
        -background $clrButtonFace -state disabled
      $base.fra31.ent38 configure \
        -background $clrButtonFace -state disabled
  } \
  else {
      $base.fra31.ent36 configure \
        -background $clrWindow -state normal
      $base.fra31.ent38 configure \
        -background $clrWindow -state normal
  }
}

proc switchVideoMode {} {
  global VideoMode clrButtonFace clrWindow
  set base [Notebook:frame .top17.note Display]

  if !$VideoMode {
      $base.fra30.03 configure \
        -background $clrButtonFace -state disabled
  } \
  else {
      $base.fra30.03 configure \
        -background $clrWindow -state normal
  }
}

proc switchAnimate {} {
global UseAnimation clrButtonFace clrWindow
set base [Notebook:frame .top17.note Animation]
  if !$UseAnimation {
    $base.fra40.01 configure -background $clrButtonFace -state disabled
    $base.fra40.06 configure -background $clrButtonFace -state disabled
    $base.fra40.07 configure -background $clrButtonFace -state disabled
    $base.fra40.010 configure -background $clrButtonFace -state disabled
    $base.fra40.011 configure -background $clrButtonFace -state disabled
    $base.fra40.012 configure -state disabled
    $base.fra40.013 configure -state disabled
    $base.fra40.014 configure -state disabled
  } \
  else {
    $base.fra40.01 configure -background $clrWindow -state normal
    $base.fra40.06 configure -background $clrWindow -state normal
    $base.fra40.07 configure -background $clrWindow -state normal
    $base.fra40.010 configure -background $clrWindow -state normal
    $base.fra40.011 configure -background $clrWindow -state normal
    $base.fra40.012 configure -state normal
    $base.fra40.013 configure -state normal
    $base.fra40.014 configure -state normal
  }
}

proc switchDisplay {} {
  global UseDisplay clrButtonFace clrWindow
  set base [Notebook:frame .top17.note Display]

  if !$UseDisplay {
    $base.fra30.04 configure -state disabled
    $base.fra30.05 configure -state disabled
    $base.fra30.06 configure -state disabled
    $base.fra30.rad17 configure -state disabled
    $base.fra30.rad18 configure -state disabled
    $base.fra31.che32 configure -state disabled
    $base.fra31.che34 configure -state disabled
    $base.fra30.03 configure -background $clrButtonFace -state disabled
    $base.fra31.ent33 configure \
     	-background $clrButtonFace -state disabled
    $base.fra31.ent36 configure \
    	-background $clrButtonFace -state disabled
    $base.fra31.ent38 configure \
    	-background $clrButtonFace -state disabled
  } \
  else {
    $base.fra30.04 configure -state normal
    $base.fra30.05 configure -state normal
    $base.fra30.06 configure -state normal
    $base.fra30.rad17 configure -state normal
    $base.fra30.rad18 configure -state normal
    $base.fra31.che32 configure -state normal
    $base.fra31.che34 configure -state normal
    $base.fra31.ent33 configure \
		-background $clrWindow -state normal
	switchGamma
	switchVideoMode
	switchMosaic
  }
}

########################################################################################
# Interface procedures
#

###################
# General Options
#
proc {PageGeneral} {Parent} {
    global clrButtonFace clrWindow
    set base [Notebook:frame $Parent General]

    label $base.lab20 \
        -borderwidth 1 -text {PovRay file}
    entry $base.ent21 \
        -textvariable FileName
    button $base.but22 \
        -command OpenFileDialog -relief groove -text ...
    label $base.lab23 \
        -borderwidth 1 -text {Output file}
    button $base.but25 \
        -command {Notebook:raise .top17.note Output} -relief groove -text >>
    frame $base.fra26 \
        -borderwidth 2 -height 75 -relief groove -width 125
    radiobutton $base.fra26.01 \
        -text {60 x 80} -value 0 -variable ImageSize
    radiobutton $base.fra26.02 \
        -text {320 x 240} -value 1 -variable ImageSize
    radiobutton $base.fra26.03 \
        -text {640 x 480} -value 2 -variable ImageSize
    radiobutton $base.fra26.04 \
        -text user -value 3 -variable ImageSize
    entry $base.fra26.05 \
        -textvariable ImgSizeX
    label $base.fra26.06 \
        -borderwidth 1 -text x
    entry $base.fra26.07 \
        -textvariable ImgSizeY
    checkbutton $base.fra26.08 \
        -justify left -text {constrain
aspect ratio} \
        -variable ConstrainAspect
    frame $base.fra27 \
        -borderwidth 2 -height 75 -relief groove -width 125
    checkbutton $base.fra27.01 \
        -anchor w -text {allow interrupt} -variable AllowInterrupt
    checkbutton $base.fra27.02 \
        -anchor w -text {continue trace} -variable ContinueTrace
    checkbutton $base.fra27.03 \
        -anchor w -command switchIni -text {create ini file} \
        -variable CreateIni
    entry $base.fra27.04 \
        -background $clrButtonFace -state disabled -textvariable IneFile
    button $base.fra27.but19 \
        -command IniFileDialog -relief groove -text ...
    checkbutton $base.fra27.05 \
        -anchor w -text {pause when done} -variable PauseWhenDone
    checkbutton $base.fra27.06 \
        -anchor w -text verbose -variable Verbose
    checkbutton $base.fra27.07 \
        -anchor w -text {draw vistas} -variable DrawVistas
    frame $base.fra28 \
        -borderwidth 2 -height 75 -relief groove -width 125
    checkbutton $base.fra28.01 \
        -anchor w -command switchPartial -text {Partial Output} \
        -variable PartialOutput
    entry $base.fra28.04 \
        -background $clrButtonFace -state disabled -textvariable PartLeft
    entry $base.fra28.03 \
        -background $clrButtonFace -state disabled -textvariable PartTop
    entry $base.fra28.02 \
        -background $clrButtonFace -state disabled -textvariable PartRight
    entry $base.fra28.05 \
        -background $clrButtonFace -state disabled -textvariable PartBottom
    label $base.fra28.07 \
        -borderwidth 1 -text Left
    label $base.fra28.06 \
        -borderwidth 1 -text Top
    label $base.fra28.08 \
        -borderwidth 1 -text Right
    label $base.fra28.09 \
        -borderwidth 1 -text Bottom
    label $base.lab17 \
        -anchor w -borderwidth 1 -relief sunken -textvariable OutputFile
    ###################
    # SETTING GEOMETRY
    ###################
    place $base.lab20 \
        -x 10 -y 10 -anchor nw -bordermode ignore
    place $base.ent21 \
        -x 75 -y 10 -width 266 -height 19 -anchor nw -bordermode ignore
    place $base.but22 \
        -x 345 -y 10 -width 15 -height 18 -anchor nw -bordermode ignore
    place $base.lab23 \
        -x 10 -y 35 -anchor nw -bordermode ignore
    place $base.but25 \
        -x 345 -y 35 -width 15 -height 18 -anchor nw -bordermode ignore
    place $base.fra26 \
        -x 5 -y 65 -width 120 -height 155 -anchor nw -bordermode ignore
    place $base.fra26.01 \
        -x 5 -y 5 -anchor nw -bordermode ignore
    place $base.fra26.02 \
        -x 5 -y 25 -anchor nw -bordermode ignore
    place $base.fra26.03 \
        -x 5 -y 45 -anchor nw -bordermode ignore
    place $base.fra26.04 \
        -x 5 -y 65 -anchor nw -bordermode ignore
    place $base.fra26.05 \
        -x 10 -y 90 -width 36 -height 19 -anchor nw -bordermode ignore
    place $base.fra26.06 \
        -x 50 -y 90 -anchor nw -bordermode ignore
    place $base.fra26.07 \
        -x 65 -y 90 -width 36 -height 19 -anchor nw -bordermode ignore
    place $base.fra26.08 \
        -x 5 -y 120 -width 88 -height 25 -anchor nw -bordermode ignore
    place $base.fra27 \
        -x 115 -y 65 -width 127 -height 155 -anchor nw -bordermode ignore
    place $base.fra27.01 \
        -x 5 -y 5 -width 98 -height 17 -anchor nw -bordermode ignore
    place $base.fra27.02 \
        -x 5 -y 25 -width 103 -height 17 -anchor nw -bordermode ignore
    place $base.fra27.03 \
        -x 5 -y 45 -width 98 -height 17 -anchor nw -bordermode ignore
    place $base.fra27.04 \
        -x 15 -y 65 -width 86 -height 19 -anchor nw -bordermode ignore
    place $base.fra27.but19 \
        -x 105 -y 65 -width 16 -height 19 -anchor nw -bordermode ignore
    place $base.fra27.05 \
        -x 5 -y 90 -width 123 -height 17 -anchor nw -bordermode ignore
    place $base.fra27.06 \
        -x 5 -y 110 -width 88 -height 17 -anchor nw -bordermode ignore
    place $base.fra27.07 \
        -x 5 -y 130 -width 88 -height 17 -anchor nw -bordermode ignore
    place $base.fra28 \
        -x 235 -y 65 -width 117 -height 155 -anchor nw -bordermode ignore
    place $base.fra28.01 \
        -x 5 -y 5 -width 98 -height 12 -anchor nw -bordermode ignore
    place $base.fra28.04 \
        -x 50 -y 25 -width 56 -height 19 -anchor nw -bordermode ignore
    place $base.fra28.03 \
        -x 50 -y 50 -width 56 -height 19 -anchor nw -bordermode ignore
    place $base.fra28.02 \
        -x 50 -y 75 -width 56 -height 19 -anchor nw -bordermode ignore
    place $base.fra28.05 \
        -x 50 -y 100 -width 56 -height 19 -anchor nw -bordermode ignore
    place $base.fra28.07 \
        -x 10 -y 25 -anchor nw -bordermode ignore
    place $base.fra28.06 \
        -x 10 -y 50 -anchor nw -bordermode ignore
    place $base.fra28.08 \
        -x 10 -y 75 -anchor nw -bordermode ignore
    place $base.fra28.09 \
        -x 10 -y 100 -anchor nw -bordermode ignore
    place $base.lab17 \
        -x 75 -y 35 -width 266 -height 17 -anchor nw -bordermode ignore
}

###################
# Display Options
#
proc {PageDisplay} {Parent} {

        global clrButtonFace clrWindow
    set base [Notebook:frame $Parent Display]
    frame $base.fra30 \
        -borderwidth 2 -height 75 -relief groove -width 125
    checkbutton $base.fra30.01 \
        -anchor w -command switchDisplay -text {display while rendering} \
        -variable UseDisplay
    label $base.fra30.02 \
        -borderwidth 1 -text {video mode}
    radiobutton $base.fra30.rad17 \
        -anchor w -command switchVideoMode -text autodetect -value 0 \
        -variable VideoMode
    radiobutton $base.fra30.rad18 \
        -anchor w -command switchVideoMode -text custom -value 1 \
        -variable VideoMode
    entry $base.fra30.03 \
        -background $clrButtonFace -state disabled \
        -textvariable CustomVideoMode
    radiobutton $base.fra30.04 \
        -anchor w -text grayscale -value 0 -variable Palette
    radiobutton $base.fra30.05 \
        -anchor w -text {16 bit} -value 1 -variable Palette
    radiobutton $base.fra30.06 \
        -anchor w -text {true color} -value 2 -variable Palette
    label $base.fra30.07 \
        -borderwidth 1 -text palette:
    frame $base.fra31 \
        -borderwidth 2 -height 75 -relief groove -width 125
    checkbutton $base.fra31.che32 \
        -anchor w -command switchGamma -text gamma -variable UseGamma
    entry $base.fra31.ent33 \
        -background $clrButtonFace -state disabled -textvariable Gamma
    checkbutton $base.fra31.che34 \
        -anchor w -command switchMosaic -text {mosaic preview} \
        -variable MosaicPreview
    label $base.fra31.lab35 \
        -borderwidth 1 -text {start size}
    entry $base.fra31.ent36 \
        -background $clrButtonFace -state disabled \
        -textvariable MosaicStartSize
    label $base.fra31.lab37 \
        -borderwidth 1 -text {end size}
    entry $base.fra31.ent38 \
        -background $clrButtonFace -state disabled \
        -textvariable MosaicEndSize
    ###################
    # SETTING GEOMETRY
    ###################
    place $base.fra30 \
        -x 5 -y 10 -width 147 -height 210 -anchor nw -bordermode ignore
    place $base.fra30.01 \
        -x 5 -y 5 -width 133 -height 17 -anchor nw -bordermode ignore
    place $base.fra30.02 \
        -x 10 -y 30 -anchor nw -bordermode ignore
    place $base.fra30.rad17 \
        -x 15 -y 50 -width 84 -height 22 -anchor nw -bordermode ignore
    place $base.fra30.rad18 \
        -x 15 -y 75 -width 70 -height 22 -anchor nw -bordermode ignore
    place $base.fra30.03 \
        -x 95 -y 75 -width 36 -height 19 -anchor nw -bordermode ignore
    place $base.fra30.04 \
        -x 15 -y 125 -width 86 -height 17 -anchor nw -bordermode ignore
    place $base.fra30.05 \
        -x 15 -y 150 -width 56 -height 17 -anchor nw -bordermode ignore
    place $base.fra30.06 \
        -x 15 -y 175 -width 91 -height 17 -anchor nw -bordermode ignore
    place $base.fra30.07 \
        -x 10 -y 105 -anchor nw -bordermode ignore
    place $base.fra31 \
        -x 150 -y 10 -width 150 -height 210 -anchor nw -bordermode ignore
    place $base.fra31.che32 \
        -x 5 -y 10 -width 63 -height 17 -anchor nw -bordermode ignore
    place $base.fra31.ent33 \
        -x 75 -y 10 -width 61 -height 19 -anchor nw -bordermode ignore
    place $base.fra31.che34 \
        -x 5 -y 40 -width 108 -height 17 -anchor nw -bordermode ignore
    place $base.fra31.lab35 \
        -x 15 -y 70 -anchor nw -bordermode ignore
    place $base.fra31.ent36 \
        -x 75 -y 70 -width 61 -height 19 -anchor nw -bordermode ignore
    place $base.fra31.lab37 \
        -x 15 -y 95 -anchor nw -bordermode ignore
    place $base.fra31.ent38 \
        -x 75 -y 95 -width 61 -height 19 -anchor nw -bordermode ignore
}


###################
# Animation Options
#
proc {PageAnimation} {Parent} {

    global clrButtonFace clrWindow
    set base [Notebook:frame $Parent Animation]
    frame $base.fra40 \
        -borderwidth 2 -height 75 -relief groove -width 125
    entry $base.fra40.01 \
        -background $clrButtonFace -state disabled -textvariable Clock
    label $base.fra40.03 \
        -borderwidth 1 -text Clock
    label $base.fra40.04 \
        -borderwidth 1 -text {Initial frame}
    label $base.fra40.05 \
        -borderwidth 1 -text {Final frame}
    entry $base.fra40.06 \
        -background $clrButtonFace -state disabled \
        -textvariable InitialFrame
    entry $base.fra40.07 \
        -background $clrButtonFace -state disabled -textvariable FinalFrame
    label $base.fra40.08 \
        -borderwidth 1 -text {Initial clock}
    label $base.fra40.09 \
        -borderwidth 1 -text {Final clock}
    entry $base.fra40.010 \
        -background $clrButtonFace -state disabled \
        -textvariable InitialClock
    entry $base.fra40.011 \
        -background $clrButtonFace -state disabled -textvariable FinalClock
    checkbutton $base.fra40.012 \
        -anchor w -state disabled -text {Cycle animation} \
        -variable CycleAnimation
    checkbutton $base.fra40.013 \
        -anchor w -state disabled -text {Field render} -variable FieldRender
    checkbutton $base.fra40.014 \
        -anchor w -state disabled -text {Odd field} -variable OddField
    checkbutton $base.che41 \
        -anchor w -command switchAnimate -text Animation \
        -variable UseAnimation
    ###################
    # SETTING GEOMETRY
    ###################
    place $base.fra40 \
        -x 5 -y 25 -width 327 -height 185 -anchor nw -bordermode ignore
    place $base.fra40.01 \
        -x 65 -y 20 -width 71 -height 19 -anchor nw -bordermode ignore
    place $base.fra40.03 \
        -x 5 -y 20 -anchor nw -bordermode ignore
    place $base.fra40.04 \
        -x 5 -y 45 -anchor nw -bordermode ignore
    place $base.fra40.05 \
        -x 5 -y 70 -anchor nw -bordermode ignore
    place $base.fra40.06 \
        -x 65 -y 45 -width 71 -height 19 -anchor nw -bordermode ignore
    place $base.fra40.07 \
        -x 65 -y 70 -width 71 -height 19 -anchor nw -bordermode ignore
    place $base.fra40.08 \
        -x 155 -y 20 -anchor nw -bordermode ignore
    place $base.fra40.09 \
        -x 155 -y 45 -anchor nw -bordermode ignore
    place $base.fra40.010 \
        -x 225 -y 20 -width 71 -height 19 -anchor nw -bordermode ignore
    place $base.fra40.011 \
        -x 225 -y 45 -width 71 -height 19 -anchor nw -bordermode ignore
    place $base.fra40.012 \
        -x 5 -y 105 -width 103 -height 17 -anchor nw -bordermode ignore
    place $base.fra40.013 \
        -x 5 -y 125 -width 88 -height 27 -anchor nw -bordermode ignore
    place $base.fra40.014 \
        -x 5 -y 150 -width 93 -height 27 -anchor nw -bordermode ignore
    place $base.che41 \
        -x 5 -y 5 -width 83 -height 12 -anchor nw -bordermode ignore
}


###################
# Output Options
#
proc {PageOutput} {Parent} {

    global clrButtonFace clrWindow
    set base [Notebook:frame $Parent Output]
    frame $base.fra17 \
        -borderwidth 2 -height 75 -relief groove -width 125
    checkbutton $base.fra17.che18 \
        -anchor w -text {don't save file} -variable DontSave
    checkbutton $base.fra17.che19 \
        -anchor w -text {output alpha} -variable OutputAlpha
    frame $base.fra43 \
        -borderwidth 2 -height 75 -relief groove -width 125
    radiobutton $base.fra43.rad44 \
        -anchor w -text {Compressed Targa (TGA)} -value 0 \
        -variable FileFormat -command setOutFile
    radiobutton $base.fra43.04 \
        -anchor w -text {Targa (TGA)} -value 1 -variable FileFormat \
        -command setOutFile
    radiobutton $base.fra43.03 \
        -anchor w -text {Portable Pixmap (PPM)} -value 2 -variable FileFormat \
        -command setOutFile
    radiobutton $base.fra43.05 \
        -anchor w -text {Portable Network Graphics (PNG)} -value 3 \
        -variable FileFormat -command setOutFile
    radiobutton $base.fra43.06 \
        -anchor w -text {System specific} -value 4 -variable FileFormat \
        -command setOutFile
    label $base.fra43.07 \
        -borderwidth 1 -text {output file type:}
    ###################
    # SETTING GEOMETRY
    ###################
    place $base.fra43 \
        -x 123 -y 10 -width 210 -height 170 -anchor nw -bordermode ignore
    place $base.fra43.rad44 \
        -x 15 -y 25 -anchor nw -bordermode ignore
    place $base.fra43.04 \
        -x 15 -y 55 -width 96 -height 17 -anchor nw -bordermode ignore
    place $base.fra43.03 \
        -x 15 -y 80 -width 146 -height 17 -anchor nw -bordermode ignore
    place $base.fra43.05 \
        -x 15 -y 105 -width 186 -height 17 -anchor nw -bordermode ignore
    place $base.fra43.06 \
        -x 15 -y 130 -width 106 -height 17 -anchor nw -bordermode ignore
    place $base.fra43.07 \
        -x 10 -y 5 -anchor nw -bordermode ignore
    place $base.fra17 \
        -x 10 -y 10 -width 115 -height 170 -anchor nw -bordermode ignore
    place $base.fra17.che18 \
        -x 5 -y 10 -width 98 -height 17 -anchor nw -bordermode ignore
    place $base.fra17.che19 \
        -x 5 -y 35 -width 88 -height 17 -anchor nw -bordermode ignore
}


###################
# Preferences
#
proc {PagePreferences} {Parent} {

    global clrButtonFace clrWindow
    set base [Notebook:frame $Parent Preferences]

    label $base.lab18 \
        -borderwidth 1 -text {POV-Ray render}
    entry $base.ent20 \
        -textvariable PovEngine
    button $base.but21 \
        -command selectPOVRender -relief groove -text ...
    label $base.lab23 \
        -borderwidth 1 -text {Include files path}
    entry $base.ent24 \
        -textvariable IncludeDir
    button $base.but25 \
        -command selectIncludePath -relief groove -text ...
    label $base.lab19 \
        -text Platform:
    label $base.lab20 \
        -textvariable Platform
    checkbutton $base.che22 \
        -anchor w -text {Keep temporary INI files} -variable che22
    button $base.but17 \
        -command savePreferences -text Save
    entry $base.ent17 \
        -textvariable DefaultDir
    button $base.but18 \
        -command selectDefaultPath -relief groove -text ...
    label $base.lab21 \
        -borderwidth 1 -text {Default project path}
    ###################
    # SETTING GEOMETRY
    ###################
    place $base.lab18 \
        -x 5 -y 5 -anchor nw -bordermode ignore
    place $base.ent20 \
        -x 5 -y 25 -width 321 -height 19 -anchor nw -bordermode ignore
    place $base.but21 \
        -x 330 -y 25 -width 15 -height 18 -anchor nw -bordermode ignore
    place $base.lab23 \
        -x 5 -y 55 -anchor nw -bordermode ignore
    place $base.ent24 \
        -x 5 -y 75 -width 321 -height 19 -anchor nw -bordermode ignore
    place $base.but25 \
        -x 330 -y 75 -width 15 -height 18 -anchor nw -bordermode ignore
    place $base.lab19 \
        -x 5 -y 220 -anchor nw -bordermode ignore
    place $base.lab20 \
        -x 65 -y 220 -width 96 -height 19 -anchor nw -bordermode ignore
    place $base.che22 \
        -x 0 -y 155 -anchor nw -bordermode ignore
    place $base.but17 \
        -x 325 -y 215 -anchor nw -bordermode ignore
    place $base.ent17 \
        -x 5 -y 125 -width 321 -height 19 -anchor nw -bordermode ignore
    place $base.but18 \
        -x 330 -y 125 -width 15 -height 18 -anchor nw -bordermode ignore
    place $base.lab21 \
        -x 5 -y 105 -width 96 -height 17 -anchor nw -bordermode ignore
}

########
# Create a new notebook widget
#
proc Notebook:create {w args} {
  global Notebook clrButtonHighlight clrButtonShadow
  set Notebook($w,width) 375
  set Notebook($w,height) 250
  set Notebook($w,pages) {}
  set Notebook($w,top) 0
  set Notebook($w,pad) 5
  set Notebook($w,fg,on) $clrButtonShadow
  set Notebook($w,fg,off) $clrButtonHighlight
  canvas $w -bd 0 -highlightthickness 0 -takefocus 0
  set Notebook($w,bg) [$w cget -bg]
  bind $w <1> "Notebook:click $w %x %y"
  bind $w <Configure> "Notebook:scheduleExpand $w"
  eval Notebook:config $w $args
}

#
# Change configuration options for the notebook widget
#
proc Notebook:config {w args} {
  global Notebook clrButtonHighlight clrButtonShadow
 									
  foreach {tag value} $args {
    switch -- $tag {
      -width {
        set Notebook($w,width) $value
      }
      -height {
        set Notebook($w,height) $value
      }
      -pages {
        set Notebook($w,pages) $value
      }
      -pad {
        set Notebook($w,pad) $value
      }
      -bg {
        set Notebook($w,bg) $value
      }
      -fg {
        set Notebook($w,fg,on) $value
      }
      -disabledforeground {
        set Notebook($w,fg,off) $value
      }
    }
  }

  #
  # After getting new configuration values, reconstruct the widget
  #
  $w delete all
  set Notebook($w,x1) $Notebook($w,pad)
  set Notebook($w,x2) [expr $Notebook($w,x1)+2]
  set Notebook($w,x3) [expr $Notebook($w,x2)+$Notebook($w,width)]
  set Notebook($w,x4) [expr $Notebook($w,x3)+2]
  set Notebook($w,y1) [expr $Notebook($w,pad)+2]
  set Notebook($w,y2) [expr $Notebook($w,y1)+2]
  set Notebook($w,y5) [expr $Notebook($w,y1)+30]
  set Notebook($w,y6) [expr $Notebook($w,y5)+2]
  set Notebook($w,y3) [expr $Notebook($w,y6)+$Notebook($w,height)]
  set Notebook($w,y4) [expr $Notebook($w,y3)+2]
  set x $Notebook($w,x1)
  set cnt 0
  set y7 [expr $Notebook($w,y1)+10]
  foreach p $Notebook($w,pages) {
    set Notebook($w,p$cnt,x5) $x
    set id [$w create text 0 0 -text $p -anchor nw -tags "p$cnt t$cnt"]
    set bbox [$w bbox $id]
    set width [lindex $bbox 2]
    $w move $id [expr $x+10] $y7
    $w create line \
       $x $Notebook($w,y5)\
       $x $Notebook($w,y2) \
       [expr $x+2] $Notebook($w,y1) \
       [expr $x+$width+16] $Notebook($w,y1) \
       -width 1 -fill $clrButtonHighlight -tags p$cnt
    $w create line \
       [expr $x+$width+16] $Notebook($w,y1) \
       [expr $x+$width+18] $Notebook($w,y2) \
       [expr $x+$width+18] $Notebook($w,y5) \
       -width 1 -fill $clrButtonShadow -tags p$cnt
    $w create line \
       [expr $x+$width+17] $Notebook($w,y1) \
       [expr $x+$width+19] $Notebook($w,y2) \
       [expr $x+$width+19] $Notebook($w,y5) \
       -width 1 -fill black -tags p$cnt
    set x [expr $x+$width+20]
    set Notebook($w,p$cnt,x6) [expr $x-2]
    if {![winfo exists $w.f$cnt]} {
      frame $w.f$cnt -bd 0
    }
    $w.f$cnt config -bg $Notebook($w,bg)
    place $w.f$cnt -x $Notebook($w,x2) -y $Notebook($w,y6) \
      -width $Notebook($w,width) -height $Notebook($w,height)
    incr cnt
  }
  $w create line \
     $Notebook($w,x1) [expr $Notebook($w,y5)-2] \
     $Notebook($w,x1) $Notebook($w,y3) \
     -width 1 -fill $clrButtonHighlight
  $w create line \
     $Notebook($w,x1) $Notebook($w,y3) \
     $Notebook($w,x2) $Notebook($w,y4) \
     $Notebook($w,x3) $Notebook($w,y4) \
     $Notebook($w,x4) $Notebook($w,y3) \
     $Notebook($w,x4) $Notebook($w,y6) \
     $Notebook($w,x3) $Notebook($w,y5) \
     -width 1 -fill $clrButtonShadow
  $w create line \
     [expr $Notebook($w,x1)+1] [expr $Notebook($w,y3)+1] \
     [expr $Notebook($w,x2)+1] [expr $Notebook($w,y4)+1] \
     [expr $Notebook($w,x3)+1] [expr $Notebook($w,y4)+1] \
     [expr $Notebook($w,x4)+1] [expr $Notebook($w,y3)+1] \
     [expr $Notebook($w,x4)+1] [expr $Notebook($w,y6)+1] \
     [expr $Notebook($w,x3)+1] [expr $Notebook($w,y5)+1] \
     -width 1 -fill black

  $w config -width [expr $Notebook($w,x4)+$Notebook($w,pad)] \
            -height [expr $Notebook($w,y4)+$Notebook($w,pad)] \
            -bg $Notebook($w,bg)
  set top $Notebook($w,top)
  set Notebook($w,top) -1
  Notebook:raise.page $w $top
}

#
# This routine is called whenever the mouse-button is pressed over
# the notebook.  It determines if any page should be raised and raises
# that page.
#
proc Notebook:click {w x y} {
  global Notebook
  if {$y<$Notebook($w,y1) || $y>$Notebook($w,y6)} return
  set N [llength $Notebook($w,pages)]
  for {set i 0} {$i<$N} {incr i} {
    if {$x>=$Notebook($w,p$i,x5) && $x<=$Notebook($w,p$i,x6)} {
      Notebook:raise.page $w $i
      break
    }
  }
}

#
# For internal use only.  This procedure raised the n-th page of
# the notebook
#
proc Notebook:raise.page {w n} {
  global Notebook clrButtonHighlight clrButtonShadow
  if {$n<0 || $n>=[llength $Notebook($w,pages)]} return
  set top $Notebook($w,top)
  if {$top>=0 && $top<[llength $Notebook($w,pages)]} {
    $w move p$top 0 2
  }
  $w move p$n 0 -2
  $w delete topline
  if {$n>0} {
    $w create line \
       $Notebook($w,x1) $Notebook($w,y6) \
       $Notebook($w,x2) $Notebook($w,y5) \
       $Notebook($w,p$n,x5) $Notebook($w,y5) \
       $Notebook($w,p$n,x5) [expr $Notebook($w,y5)-2] \
       -width 2 -fill $clrButtonHighlight -tags topline
  }
  $w create line \
    $Notebook($w,p$n,x6) [expr $Notebook($w,y5)-2] \
    $Notebook($w,p$n,x6) $Notebook($w,y5) \
    -width 2 -fill $clrButtonHighlight -tags topline
  $w create line \
    $Notebook($w,p$n,x6) $Notebook($w,y5) \
    $Notebook($w,x3) $Notebook($w,y5) \
    -width 2 -fill $clrButtonHighlight -tags topline
  set Notebook($w,top) $n
  raise $w.f$n
}

#
# Change the page-specific configuration options for the notebook
#
proc Notebook:pageconfig {w name args} {
  global Notebook
  set i [lsearch $Notebook($w,pages) $name]
  if {$i<0} return
  foreach {tag value} $args {
    switch -- $tag {
      -state {
        if {"$value"=="disabled"} {
          $w itemconfig t$i -fg $Notebook($w,fg,off)
        } else {
          $w itemconfig t$i -fg $Notebook($w,fg,on)
        }
      }
      -onexit {
        set Notebook($w,p$i,onexit) $value
      }
    }
  }
}

#
# This procedure raises a notebook page given its name.  But first
# we check the "onexit" procedure for the current page (if any) and
# if it returns false, we don't allow the raise to proceed.
#
proc Notebook:raise {w name} {
  global Notebook
  set i [lsearch $Notebook($w,pages) $name]
  if {$i<0} return
  if {[info exists Notebook($w,p$i,onexit)]} {
    set onexit $Notebook($w,p$i,onexit)
    if {"$onexit"!="" && [eval uplevel #0 $onexit]!=0} {
      Notebook:raise.page $w $i
    }
  } else {
    Notebook:raise.page $w $i
  }
}

#
# Return the frame associated with a given page of the notebook.
#
proc Notebook:frame {w name} {
  global Notebook
  set i [lsearch $Notebook($w,pages) $name]
  if {$i>=0} {
    return $w.f$i
  } else {
    return {}
  }
}

#
# Try to resize the notebook to the next time we become idle.
#
proc Notebook:scheduleExpand w {
  global Notebook
  if {[info exists Notebook($w,expand)]} return
  set Notebook($w,expand) 1
  after idle "Notebook:expand $w"
}

#
# Resize the notebook to fit inside its containing widget.
#
proc Notebook:expand w {
  global Notebook
  set wi [expr [winfo width $w]-($Notebook($w,pad)*2+4)]
  set hi [expr [winfo height $w]-($Notebook($w,pad)*2+36)]
  Notebook:config $w -width $wi -height $hi
  catch {unset Notebook($w,expand)}
}


proc init {argc argv} {

}

init $argc $argv

proc {main} {argc argv} {

global Platform tcl_platform clrButtonFace clrWindow clrButtonShadow clrButtonHighlight
	set Platform $tcl_platform(platform)
    bind .top17 <Alt-s> { exit }
    readPreferences
    if {$argc==1} {
    	if {[file exists $argv]}  {
    		set FileName $argv
    	}
    }
	readPreferences
	focus -force .top17
}

proc {Window} {args} {

    global vTcl
    set cmd [lindex $args 0]
    set name [lindex $args 1]
    set newname [lindex $args 2]
    set rest [lrange $args 3 end]
    if {$name == "" || $cmd == ""} {return}
    if {$newname == ""} {
        set newname $name
    }
    set exists [winfo exists $newname]
    switch $cmd {
        show {
            if {$exists == "1" && $name != "."} {wm deiconify $name; return}
            if {[info procs vTclWindow(pre)$name] != ""} {
                eval "vTclWindow(pre)$name $newname $rest"
            }
            if {[info procs vTclWindow$name] != ""} {
                eval "vTclWindow$name $newname $rest"
            }
            if {[info procs vTclWindow(post)$name] != ""} {
                eval "vTclWindow(post)$name $newname $rest"
            }
        }
        hide    { if $exists {wm withdraw $newname; return} }
        iconify { if $exists {wm iconify $newname; return} }
        destroy { if $exists {destroy $newname; return} }
    }
}

#################################
# VTCL GENERATED GUI PROCEDURES
#

proc vTclWindow. {base} {
    if {$base == ""} {
        set base .
    }
    ###################
    # CREATING WIDGETS
    ###################
    wm focusmodel $base passive
    wm geometry $base 200x200+0+0
    wm maxsize $base 804 585
    wm minsize $base 100 1
    wm overrideredirect $base 0
    wm resizable $base 1 1
    wm withdraw $base
    wm title $base "vt"
    ###################
    # SETTING GEOMETRY
    ###################
}

proc vTclWindow.top17 {base} {
    if {$base == ""} {
        set base .top17
    }
    if {[winfo exists $base]} {
        wm deiconify $base; return
    }
    ###################
    # CREATING WIDGETS
    ###################
    toplevel $base -class Toplevel
    wm focusmodel $base passive
    wm geometry $base 462x309+234+99
    wm maxsize $base 804 585
    wm minsize $base 100 1
    wm overrideredirect $base 0
    wm resizable $base 0 0
    wm deiconify $base
    wm title $base "POVer - Preview 1.02"
    wm protocol $base WM_DELETE_WINDOW exit
    button $base.but18 \
        -command TraceIt -text Trace
    button $base.but17 \
        -command OpenIt -text Open
    button $base.but19 \
        -command SaveIt -text Save
    button $base.but20 \
        -command exit -text Quit
    Notebook:create $base.note -pages {General Display Animation Output Preferences} -pad 8
    ######## Copy widgets in this place
    ###################
    # SETTING GEOMETRY
    ###################

    place $base.note \
        -x 3 -y 3 -anchor nw -bordermode ignore
    place $base.but18 \
        -x 405 -y 10 -width 46 -height 28 -anchor nw -bordermode ignore
    place $base.but17 \
        -x 405 -y 45 -width 46 -height 28 -anchor nw -bordermode ignore
    place $base.but19 \
        -x 405 -y 80 -width 45 -height 28 -anchor nw -bordermode ignore
    place $base.but20 \
        -x 405 -y 115 -width 45 -height 28 -anchor nw -bordermode ignore
    ####### Copy config it in this place
    PageGeneral   $base.note
    PageDisplay   $base.note
    PageAnimation $base.note
    PageOutput    $base.note
    PagePreferences $base.note
}

# To be compatible with Windows Themes
if {"$tcl_platform(platform)"=="windows"} {
		set clrButtonFace 			{SystemButtonFace}
		set clrWindow 				{SystemWindow}
		set clrButtonShadow 	{SystemButtonShadow}
		set clrButtonHighlight 	{SystemButtonHighlight}
} else {
    	set clrButtonFace 			{gray}
    	set clrWindow 				{white}
    	set clrButtonShadow 	{darkgray}
    	set clrButtonHighlight 	{lightgray}
}

Window show .
Window show .top17

main $argc $argv